__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"


def frozen_graph(
        model_frozen_fpn='./frozen_model.pb'
):
    """
    You must use this function using TensorFlow 2x series.

    Parameters
    ----------
    pb_file_path

    Returns
    -------

    """
    import tensorflow as tf

    with tf.io.gfile.GFile(model_frozen_fpn, "rb") as f:
        graph_def = tf.compat.v1.GraphDef()
        graph_def.ParseFromString(f.read())

    def wrap_graph():
        tf.compat.v1.import_graph_def(graph_def, name="")

    tf.compat.v1.reset_default_graph()
    wrap_graph()
    graph = tf.compat.v1.get_default_graph()
    return graph


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"


class Index:

    def __init__(self, sequence):
        self.sequence = sequence
        self.len_seq = len(self.sequence)

    def get(self):
        ids = []
        for id in range(self.len_seq):
            ids.append(id + 1)
        return ids


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import numpy as np


class DataInitializer:

    def input2d(self, data, bound_inf=None, bound_sup=None, ):
        shape = data.shape
        num_samples = shape[0]
        num_features = shape[1]
        if bound_inf is not None and bound_sup is not None:
            x_data = data[:, 0: bound_inf].astype(np.float32)
            y_data = data[:, bound_sup: num_features].astype(np.float32)
            return x_data, y_data, num_samples
        elif bound_inf is None and bound_sup is None:
            return data, num_samples

    def batchData(self, x_data, y_data, totem, batch_size, y_dim=2):
        x_batch_data = x_data[totem*batch_size:(totem+1)*batch_size, :]
        if y_dim == 2:
            y_batch_data = y_data[totem*batch_size:(totem+1)*batch_size, :]
        else:
            y_batch_data = y_data[totem*batch_size:(totem+1)*batch_size]
        return x_batch_data, y_batch_data

    def testData(self, x_test, y_test, truncation_inf, truncation_sup, y_dim=2):
        x_data = x_test[truncation_inf:truncation_sup, :]
        if y_dim == 2:
            y_data = y_test[truncation_inf:truncation_sup, :]
        else:
            y_data = y_test[truncation_inf:truncation_sup]
        return x_data, y_data


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"


class BPair:

    def __init__(self, sequence, window_size, window_aa_ids):
        self.sequence = sequence
        self.window_size = window_size
        self.window_aa_ids = window_aa_ids
        self.num_pairs = len(self.window_aa_ids)
        self.stretch_window = int((self.window_size * 2 + 1) * (self.window_size * 2) / 2)

    def patch(self, length, step=1):
        arr = []
        for i in range(-length, length + 1, step):
            for j in range(-length, length + 1, step):
                arr.append([i, j])
        return arr


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import pandas as pd
from deephelicon.util.Position import Position
from deephelicon.util.Separation import Separation


class Length(Position):

    def __init__(self, seq_sep_inferior=None, seq_sep_superior=None):
        super(Length, self).__init__(seq_sep_inferior, seq_sep_superior)

    def num2arr(self, length):
        arr = []
        for i in range(length):
            for j in range(length):
                if i < j:
                    arr.append([i + 1, j + 1])
        return arr

    def toPair(self, length):
        df_ = pd.DataFrame(self.num2arr(length))
        pairs = Separation(
            df=df_,
            first=0,
            second=1,
            is_sort=False,
            seq_sep_inferior=self.seq_sep_inferior,
            seq_sep_superior=self.seq_sep_superior
        ).extract().values.tolist()
        return pairs


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import numpy as np
from deephelicon.util.Plmc import Plmc as cmplmc
from deephelicon.util.Irreflexive import Irreflexive
from deephelicon.util.PFasta import PFasta as pfasta
from deephelicon.util.Length import Length as plength
from deephelicon.util.Pair import Pair
from deephelicon.util.Console import Console


class FeatureS1:

    def __init__(
            self,
            params,
            verbose=True,
    ):
        self.params = params

        self.console = Console()
        self.console.verbose = verbose

    def offone(
            self,
            prot_name,
            file_chain,
            sequence,
            position,
            window_aa_ids,
    ):
        tds = [[] for _ in range(len(position))]
        ecps = cmplmc()
        tds = ecps.Jij(
            list_2d=tds,
            position=position,
            prot_name=prot_name,
            file_chain=file_chain,
            param_path=self.params['plmc_param_fp'],
        )
        tds = Irreflexive(
            sequence=sequence,
            window_aa_ids=window_aa_ids,
            window_size=1,
        ).assign(
            list_2d=tds,
            prot_name=prot_name,
            file_chain=file_chain,
            dhc_suffix=None,
            fc_path=self.params['fc_fp'],
            cp_path=self.params['cp_fp'],
            plmc_path=self.params['plmc_fp'],
            gdca_path=self.params['gdca_fp']
        )
        return tds

    def generate(self):
        self.console.print('======>Features are being assembled...')
        pos_list = plength(4).toPair(len(self.params['sequence']))
        num_samples = len(pos_list)
        position = pfasta(self.params['sequence']).pair(pos_list=pos_list)
        window_aa_ids = Pair(
            sequence=self.params['sequence'],
            position=position,
            window_size=1,
        ).aaid()
        fts = self.offone(
            prot_name=self.params['prot_name'],
            file_chain=self.params['file_chain'],
            sequence=self.params['sequence'],
            position=position,
            window_aa_ids=window_aa_ids
        )
        rff = np.array(fts)[:, 0: 728].astype(np.float64)
        lll = np.array(len(position) * [0]).astype(np.int64)
        ocs = np.zeros((num_samples, 2))
        ocs[np.arange(num_samples), lll] = 1
        ofd = np.concatenate((rff, ocs), axis=1)
        if self.params['sv_fp']:
            f = open(self.params['sv_fp'] + self.params['prot_name'] + self.params['file_chain'] + self.params['sv_suffix'], 'w')
            for i in ofd:
                k = ' '.join([str(j) for j in i])
                f.write(k + "\n")
            f.close()
        self.console.print('======>Finished assembling features.')
        return ofd


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"


class Position:

    def __init__(self, seq_sep_inferior=None, seq_sep_superior=None):
        self.seq_sep_inferior = seq_sep_inferior
        self.seq_sep_superior = seq_sep_superior


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

from Bio import SeqIO


class Fasta:

    def get(self, fasta_fpn):
        sequence = []
        for seq in SeqIO.parse(fasta_fpn, "fasta"):
            sequence.append(str(seq.seq))
        sequence = ''.join(sequence)
        if sequence == '':
            print('The sequence is empty.')
        return sequence


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"


class Pair:

    def __init__(self, sequence, position, window_size):
        self.sequence = sequence
        self.aa_pairs = position
        self.window_size = window_size
        self.len_seq = len(self.sequence)
        self.num_pairs = len(self.aa_pairs)

    def aaid(self):
        window_aa_ids = [[[] for _ in range(2)] for _ in range(self.num_pairs)]
        for i in range(self.num_pairs):
            for aa1_left_id in range(self.window_size):
                window_aa_ids[i][0].append(self.aa_pairs[i][0] - (aa1_left_id + 1))
            for aa1_right_id in range(self.window_size):
                window_aa_ids[i][0].append(self.aa_pairs[i][0] + (aa1_right_id + 1))
            window_aa_ids[i][0].append(self.aa_pairs[i][0])

        for i in range(self.num_pairs):
            for aa2_left_id in range(self.window_size):
                window_aa_ids[i][1].append(self.aa_pairs[i][3] - (aa2_left_id + 1))
            for aa2_right_id in range(self.window_size):
                window_aa_ids[i][1].append(self.aa_pairs[i][3] + (aa2_right_id + 1))
            window_aa_ids[i][1].append(self.aa_pairs[i][3])
        for i in range(self.num_pairs):
            for j in range(self.window_size * 2 + 1):
                if window_aa_ids[i][0][j] < 1 or window_aa_ids[i][0][j] > self.len_seq:
                    window_aa_ids[i][0][j] = None
                if window_aa_ids[i][1][j] < 1 or window_aa_ids[i][1][j] > self.len_seq:
                    window_aa_ids[i][1][j] = None
        return window_aa_ids


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import click
import urllib.request
from pyfiglet import Figlet
import numpy as np
import pandas as pd
from deephelicon import load
from deephelicon.util.Fasta import Fasta as sfasta
from deephelicon.util.PFasta import PFasta as pfasta
from deephelicon.util.Length import Length as sclength
from deephelicon.util.DataInitializer import DataInitializer
from deephelicon.util.FeatureS1 import FeatureS1
from deephelicon.util.FeatureS2 import FeatureS2
from deephelicon.util.Console import Console


vignette1 = Figlet(font='standard')

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.command(short_help=vignette1.renderText('DeepHelicon'), context_settings=CONTEXT_SETTINGS)
@click.option('-u', '--url', default='https://github.com/2003100127/deephelicon/releases/download/model/model.zip', help='URL of deephelicon models')
@click.option('-o', '--sv_fpn', default='./model.zip', help='output path of deephelicon models')
def download(url, sv_fpn):
    download_data(url, sv_fpn)


def download_data(
        url,
        sv_fpn,
        verbose=True,
):
    console = Console()
    console.verbose = verbose
    print(vignette1.renderText('DeepHelicon'))
    console.print('=>Downloading starts...')
    urllib.request.urlretrieve(
        url=url,
        filename=sv_fpn
    )
    console.print('=>downloaded.')
    return 'downloaded.'


class HelpfulCmd(click.Command):
    def format_help(self, ctx, formatter):
        click.echo(vignette1.renderText('DeepHelicon'))
        click.echo(
            '''
            Options:
            -pn, --prot_name              Protein name identifier (e.g., "5w1h")
            -pc, --prot_chain             Protein chain identifier (e.g., "A")
            -fa, --fasta_fp               Path to the FASTA file containing the protein sequence
            -fcfp, --fc_fp                Path to the FreeContact feature file
            -gdcafp, --gdca_fp            Path to the Gaussian DCA feature file
            -cpfp, --cp_fp                Path to the CCMPred feature file
            -plmcfp, --plmc_fp            Path to the PLMC coupling score file
            -plmcparamfp, --plmc_param_fp Path to the PLMC parameter file
            -sv_fp_f, --sv_fp_feature     Path to save the generated input features for the model (default: ./)
            -sv_suf_f, --sv_suffix_feature    Suffix name for the saved feature file (e.g., ".fs1", default: .fs1)
            -m, --model_frozen_fpn        Path to the frozen model (.pb) file for prediction
            -sv_fp_p, --sv_fp_pred        Path to save the model prediction outputs (default: ./)
            -sv_suf_p, --sv_suffix_pred   Suffix name for the saved prediction file (e.g., ".s1", default: .s1)
            -ss_inf, --seq_sep_inferior   Sequence separation inferior limit (default: 4)
            -bs, --batch_size             Batch size used during prediction (default: 100)
            -vb, --verbose                Whether to print detailed logs during processing (default: True)
            '''
        )


@click.command(cls=HelpfulCmd, context_settings=CONTEXT_SETTINGS)
@click.option('-pn', '--prot_name', required=True, help='Protein name identifier (e.g., "5w1h")')
@click.option('-pc', '--prot_chain', required=True, help='Protein chain identifier (e.g., "A")')
@click.option('-fa', '--fasta_fp', required=True, help='Path to the FASTA file containing the protein sequence')
@click.option('-fcfp', '--fc_fp', required=True, help='Path to the freecontact feature file')
@click.option('-gdcafp', '--gdca_fp', required=True, help='Path to the Gaussion DCA feature file')
@click.option('-cpfp', '--cp_fp', required=True, help='Path to the CCMPred feature file')
@click.option('-plmcfp', '--plmc_fp', required=True, help='Path to the PLMC coupling score file')
@click.option('-plmcparamfp', '--plmc_param_fp', required=True, help='Path to the PLMC parameter file')
@click.option('-sv_fp_f', '--sv_fp_feature', default='./', help='Path to save the generated input features for the model')
@click.option('-sv_suf_f', '--sv_suffix_feature', default='.fs1', help='Suffix name for the saved feature file (e.g., ".fs1")')
@click.option('-m', '--model_frozen_fpn', required=True, help='Path to the frozen model (.pb) file for prediction')
@click.option('-sv_fp_p', '--sv_fp_pred', default='./', help='Path to save the model prediction outputs')
@click.option('-sv_suf_p', '--sv_suffix_pred', default='.s1', help='Suffix name for the saved prediction file (e.g., "_pred.csv")')
@click.option('-ss_inf', '--seq_sep_inferior', default=4, help='sequence separation inferior')
@click.option('-bs', '--batch_size', default=100, help='Batch size used during prediction')
@click.option('-vb', '--verbose', default=True, help='Whether to print detailed logs during processing (default: True)')
def rrc_s1(
        prot_name,
        prot_chain,
        fasta_fp,
        fc_fp,
        gdca_fp,
        cp_fp,
        plmc_fp,
        plmc_param_fp,
        sv_fp_feature,
        sv_suffix_feature,
        model_frozen_fpn,
        sv_fp_pred,
        sv_suffix_pred,
        seq_sep_inferior,
        batch_size,
        verbose,
):
    return rrcontact_stage1(
        prot_name=prot_name,
        prot_chain=prot_chain,
        fasta_fp=fasta_fp,
        fc_fp=fc_fp,
        gdca_fp=gdca_fp,
        cp_fp=cp_fp,
        plmc_fp=plmc_fp,
        plmc_param_fp=plmc_param_fp,
        sv_fp_feature=sv_fp_feature,
        sv_suffix_feature=sv_suffix_feature,
        model_frozen_fpn=model_frozen_fpn,
        sv_fp_pred=sv_fp_pred,
        sv_suffix_pred=sv_suffix_pred,
        seq_sep_inferior=seq_sep_inferior,
        batch_size=batch_size,
        verbose=verbose,
)


def rrcontact_stage1(
        prot_name,
        prot_chain,
        fasta_fp,
        fc_fp,
        gdca_fp,
        cp_fp,
        plmc_fp,
        plmc_param_fp,
        sv_fp_feature,
        sv_suffix_feature,
        model_frozen_fpn,
        sv_fp_pred,
        sv_suffix_pred,
        seq_sep_inferior=5,
        batch_size=100,
        verbose=True,
):
    console = Console()
    console.verbose = verbose
    print(vignette1.renderText('DeepHelicon'))

    console.print("===>Protein: {} chain: {}".format(prot_name, prot_chain))

    sequence = sfasta().get(fasta_fpn=fasta_fp + prot_name + prot_chain + '.fasta')

    mat_np = FeatureS1(params={
        'prot_name': prot_name,
        'file_chain': prot_chain,
        'sequence': sequence,
        'fc_fp': fc_fp,
        'gdca_fp': gdca_fp,
        'plmc_fp': plmc_fp,
        'cp_fp': cp_fp,
        'plmc_param_fp': plmc_param_fp,
        'sv_fp': sv_fp_feature,
        'sv_suffix': sv_suffix_feature,
    }).generate()
    # print(mat_np)
    x_test, _, num_test_samples = DataInitializer().input2d(
        mat_np,
        bound_inf=728,
        bound_sup=728,
    )

    import tensorflow as tf
    console.print("===>The model ({}) with frozen graphs converted from tensorflow 1.15.2 is being read...".format(model_frozen_fpn))
    graph = load.frozen_graph(model_frozen_fpn)
    x_tf = graph.get_tensor_by_name("x:0")
    pred_tf = graph.get_tensor_by_name("softmax:0")
    console.print("===>The model is read in, with\ninput: {}\noutput: {}".format(x_tf, pred_tf))
    sess = tf.compat.v1.Session(graph=graph)

    accumulator = []

    num_batch_test = num_test_samples // batch_size
    final_number = num_test_samples % batch_size
    for batch in range(num_batch_test + 1):
        if batch < num_batch_test:
            x_batch_te, _ = DataInitializer().batchData(
                x_test, _, batch, batch_size
            )
        else:
            x_batch_te = x_test[batch * batch_size: (batch * batch_size + final_number), :]
        feed_dict_test = {x_tf: x_batch_te}
        pred_tmp = sess.run(pred_tf, feed_dict=feed_dict_test)
        accumulator.append(pred_tmp)

    pos_list = sclength(seq_sep_inferior).toPair(len(sequence))
    position = pfasta(sequence).pair(pos_list=pos_list)
    res = np.array(position)[:, [0, 1, 3, 4]]
    pred_data = accumulator[0]
    for i in range(1, len(accumulator)):
        pred_data = np.concatenate((pred_data, accumulator[i]), axis=0)
    pred_data = np.concatenate((res, pred_data[:, [1]]), axis=1)

    df = pd.DataFrame(pred_data)
    console.print("======>Predicted probabilities of residue-residue contacts:\n{}".format(df))
    if sv_fp_pred:
        sv_fpn_pred = sv_fp_pred + prot_name + prot_chain + sv_suffix_pred
        df.to_csv(
            sv_fpn_pred,
            sep='\t',
            header=False,
            index=False
        )
        console.print("======>Predictions are saved to {}".format(sv_fpn_pred))
    return df


class HelpfulCmd2(click.Command):
    def format_help(self, ctx, formatter):
        click.echo(vignette1.renderText('DeepHelicon'))
        click.echo(
            '''
            Options:
            -pn, --prot_name              Protein name identifier (e.g., "5w1h")
            -pc, --prot_chain             Protein chain identifier (e.g., "A")
            -fa, --fasta_fp               Path to the FASTA file containing the protein sequence
            -dhcfp, --dhc_fp              Path to save a residue-residue contact file
            -dhc_suf, --dhc_suffix        Suffix name for the saved residue-residue contact prediction file
                                        (e.g., ".s1" or ".s2i1")
            -sv_fp_f, --sv_fp_feature     Path to save the generated structural feature file for the model
                                        (default: "./")
            -sv_suf_f, --sv_suffix_feature
                                        Suffix name for the saved structural feature file
                                        (e.g., ".s2i2", default: ".fs2i1")
            -m, --model_frozen_fpn        Path to the frozen model (.pb file) used for prediction
            -sv_fp_p, --sv_fp_pred        Path to save the model prediction output (default: "./")
            -sv_suf_p, --sv_suffix_pred   Suffix name for the saved prediction file
                                        (e.g., "_pred.csv", default: ".s2i1")
            -ss_inf, --seq_sep_inferior   Inferior threshold for sequence separation filtering
                                        (residue pairs partitioned less than this number will be ignored, default: 5)
            -bs, --batch_size             Batch size used during model inference (default: 100)
            -vb, --verbose                Whether to print detailed logs during processing (default: True)
            '''
        )


@click.command(cls=HelpfulCmd2, context_settings=CONTEXT_SETTINGS)
@click.option('-pn', '--prot_name', required=True, help='Protein name identifier (e.g., "5w1h")')
@click.option('-pc', '--prot_chain', required=True, help='Protein chain identifier (e.g., "A")')
@click.option('-fa', '--fasta_fp', required=True, help='Path to the FASTA file containing the protein sequence')
@click.option('-dhcfp', '--dhc_fp', required=True, help='Path to save a residue-residue contact file')
@click.option('-dhc_suf', '--dhc_suffix', required=True, help='Suffix name for the saved residue-residue contact prediction file (e.g., ".s1" or ".s2i1")')
@click.option('-sv_fp_f', '--sv_fp_feature', default='./', help='Path to save the generated structural feature file for the model')
@click.option('-sv_suf_f', '--sv_suffix_feature', default='.fs2i1', help='Suffix name for the saved structural feature file (e.g., ".s2i2")')
@click.option('-m', '--model_frozen_fpn', required=True, help='Path to the frozen model (.pb file) used for prediction')
@click.option('-sv_fp_p', '--sv_fp_pred', default='./', help='Path to save the model prediction output')
@click.option('-sv_suf_p', '--sv_suffix_pred', default='.s2i1', help='Suffix name for the saved prediction file (e.g., "_pred.csv")')
@click.option('-ss_inf', '--seq_sep_inferior', default=5, help='Inferior threshold for sequence separation filtering (e.g., residue pairs partitioned less than this number will be ignored)')
@click.option('-bs', '--batch_size', default=100, help='Batch size used during model inference')
@click.option('-vb', '--verbose', default=True, help='Whether to print detailed logs during processing (default: True)')
def rrc_s2(
        prot_name,
        prot_chain,
        fasta_fp,
        dhc_fp,
        dhc_suffix,
        sv_fp_feature,
        sv_suffix_feature,
        model_frozen_fpn,
        sv_fp_pred,
        sv_suffix_pred,
        seq_sep_inferior,
        batch_size,
        verbose,
):
    return rrcontact_stage2(
        prot_name=prot_name,
        prot_chain=prot_chain,
        fasta_fp=fasta_fp,
        dhc_fp=dhc_fp,
        dhc_suffix=dhc_suffix,
        sv_fp_feature=sv_fp_feature,
        sv_suffix_feature=sv_suffix_feature,
        model_frozen_fpn=model_frozen_fpn,
        sv_fp_pred=sv_fp_pred,
        sv_suffix_pred=sv_suffix_pred,
        seq_sep_inferior=seq_sep_inferior,
        batch_size=batch_size,
        verbose=verbose,
    )


def rrcontact_stage2(
        prot_name,
        prot_chain,
        fasta_fp,
        dhc_fp,
        dhc_suffix,

        sv_fp_feature,
        sv_suffix_feature,

        model_frozen_fpn,

        sv_fp_pred,
        sv_suffix_pred,

        seq_sep_inferior,
        batch_size=100,
        verbose=True,
):
    console = Console()
    console.verbose = verbose
    print(vignette1.renderText('DeepHelicon'))

    console.print("===>Protein: {} chain: {}".format(prot_name, prot_chain))

    sequence = sfasta().get(fasta_fpn=fasta_fp + prot_name + prot_chain + '.fasta')

    mat_np = FeatureS2(params={
        'prot_name': prot_name,
        'file_chain': prot_chain,
        'sequence': sequence,
        'dhc_fp': dhc_fp,
        'dhc_suffix': dhc_suffix,
        'sv_fp': sv_fp_feature,
        'sv_suffix': sv_suffix_feature,
    }).generate()
    # print(mat_np)
    x_test, _, num_test_samples = DataInitializer().input2d(
        mat_np,
        bound_inf=225,
        bound_sup=225,
    )

    import tensorflow as tf
    console.print("===>The model ({}) with frozen graphs converted from tensorflow 1.15.2 is being read...".format(model_frozen_fpn))
    graph = load.frozen_graph(model_frozen_fpn)
    x_tf = graph.get_tensor_by_name("x_1:0")
    pred_tf = graph.get_tensor_by_name("sigmoid:0")
    console.print("===>The model is read in, with\ninput: {}\noutput: {}".format(x_tf, pred_tf))
    sess = tf.compat.v1.Session(graph=graph)

    accumulator = []
    num_batch_test = num_test_samples // batch_size
    final_number = num_test_samples % batch_size
    for batch in range(num_batch_test + 1):
        if batch < num_batch_test:
            x_batch_te, _ = DataInitializer().batchData(
                x_test, _, batch, batch_size
            )
        else:
            x_batch_te = x_test[batch * batch_size: (batch * batch_size + final_number), :]
        feed_dict_test = {x_tf: x_batch_te}
        pred_tmp = sess.run(pred_tf, feed_dict=feed_dict_test)
        accumulator.append(pred_tmp)
    pos_list = sclength(int(seq_sep_inferior)).toPair(len(sequence))
    position = pfasta(sequence).pair(pos_list=pos_list)
    res = np.array(position)[:, [0, 1, 3, 4]]
    pred_data = accumulator[0]
    for i in range(1, len(accumulator)):
        pred_data = np.concatenate((pred_data, accumulator[i]), axis=0)
    pred_data = np.concatenate((res, pred_data[:, [0]]), axis=1)
    df = pd.DataFrame(pred_data)
    console.print("======>Predicted probabilities of residue-residue contacts:\n{}".format(df))
    if sv_fp_pred:
        sv_fpn_pred = sv_fp_pred + prot_name + prot_chain + sv_suffix_pred
        df.to_csv(
            sv_fpn_pred,
            sep='\t',
            header=False,
            index=False
        )
        console.print("======>Predictions are saved to {}".format(sv_fpn_pred))
    return df


class HelpfulCmdr(click.Command):
    def format_help(self, ctx, formatter):
        click.echo(vignette1.renderText('DeepHelicon'))
        click.echo(
            '''
            Options:
            -pn, --prot_name    Protein name identifier (e.g., "5w1h")
            -pc, --prot_chain   Protein chain identifier (e.g., "A")
            -fps, --file_paths  Python dict of input file paths. The files are produced by DeepHelicon at stage 1 and stage 2.
            -fss, --file_suffixs    Python dict of corresponding suffixes for each of the input files (file_paths)
            -fids, --file_ids   IDs of input files
            -tmhmm, --tmhmm_fp  Path to the TMHMM prediction file for transmembrane region annotation
            -sv_fp, --sv_fp Path to save the merged or processed feature file (e.g., .s1, s2i1, s2i2, etc.) (default: ./)
            -fmt, --format_rr   Format of the output file for residue-residue contact predictions. The default is "Normal", but "CASP14" is also supported.
            -vb, --verbose  Whether to print detailed logs during processing (default: True)
            '''
        )


@click.command(cls=HelpfulCmdr, context_settings=CONTEXT_SETTINGS)
@click.option('-pn', '--prot_name', required=True, help='Protein name identifier (e.g., "5w1h")')
@click.option('-pc', '--prot_chain', required=True, help='Protein chain identifier (e.g., "A")')
@click.option('-fps', '--file_paths', required=True, help='Python dict of input file paths. The files are producted by DeepHelicon at stage 1 and stage 2.')
@click.option('-fss', '--file_suffixs', required=True, help='Python dict of corresponding suffixes for each of the input files (file_paths)')
@click.option('-fids', '--file_ids', required=True, help='IDs of input files')
@click.option('-tmhmm', '--tmhmm_fp', required=True, help='Path to the TMHMM prediction file for transmembrane region annotation')
@click.option('-sv_fp', '--sv_fp', default='./', help='Path to save the merged or processed feature file (e.g., .s1, s2i1, s2i2, etc.)')
@click.option('-fmt', '--format_rr', default='Normal', help='Format of the output file for residue-residue contact predictions. The default option is Normal but it can be CASP14.')
@click.option('-vb', '--verbose', default=True, help='Whether to print detailed logs during processing (default: True)')
def op_format(
        prot_name,
        prot_chain,
        file_paths,
        file_suffixs,
        file_ids,
        tmhmm_fp,
        sv_fp,
        format_rr,
        verbose,
):
    return reformat(
        prot_name=prot_name,
        prot_chain=prot_chain,
        file_paths=file_paths,
        file_suffixs=file_suffixs,
        file_ids=file_ids,
        tmhmm_fp=tmhmm_fp,
        sv_fp=sv_fp,
        format_rr=format_rr,
        verbose=verbose,
    )


def reformat(
        prot_name,
        prot_chain,
        file_paths,
        file_suffixs,
        file_ids,
        tmhmm_fp,
        sv_fp,
        format_rr,
        verbose=True,
):
    console = Console()
    console.verbose = verbose
    print(vignette1.renderText('DeepHelicon'))

    from deephelicon.util.Reformat import Reformat
    df = Reformat().run(
        prot_name=prot_name,
        file_chain=prot_chain,
        file_paths=file_paths,
        file_suffixs=file_suffixs,
        file_ids=file_ids,
        tmhmm_fp=tmhmm_fp,
        sv_fp=sv_fp,
        format_rr=format_rr,
    )
    console.print("dataframe: \n{}".format(df))
    return df


if __name__ == "__main__":
    # params_s1 = {
    #     'prot_name': '2wsc',
    #     'prot_chain': '2',
    #     'fasta_fp': '../data/input/',
    #     'fc_fp': '../data/input/',
    #     'gdca_fp': '../data/input/',
    #     'cp_fp': '../data/input/',
    #     'plmc_fp': '../data/input/',
    #     'plmc_param_fp': '../data/input/',
    #     'sv_fp_feature': '../data/',
    #     'sv_suffix_feature': '.fs1',
    #
    #     'model_frozen_fpn': '../data/model/frozen_graph/s1.pb',
    #     'batch_size': 100,
    #     'seq_sep_inferior': 4,
    #     'sv_fp_pred': '../data/output/test/',
    #     'sv_suffix_pred': '.s1',
    # }
    # rrcontact_stage1(
    #     prot_name=params_s1['prot_name'],
    #     prot_chain=params_s1['prot_chain'],
    #     fasta_fp=params_s1['fasta_fp'],
    #     fc_fp=params_s1['fc_fp'],
    #     gdca_fp=params_s1['gdca_fp'],
    #     cp_fp=params_s1['cp_fp'],
    #     plmc_fp=params_s1['plmc_fp'],
    #     plmc_param_fp=params_s1['plmc_param_fp'],
    #     sv_fp_feature=params_s1['sv_fp_feature'],
    #     sv_suffix_feature=params_s1['sv_suffix_feature'],
    #     model_frozen_fpn=params_s1['model_frozen_fpn'],
    #     sv_fp_pred=params_s1['sv_fp_pred'],
    #     sv_suffix_pred=params_s1['sv_suffix_pred'],
    #     seq_sep_inferior=params_s1['seq_sep_inferior'],
    #     batch_size=100,
    #     verbose=True,
    # )

    # params_s2i1 = {
    #     'prot_name': '2wsc',
    #     'prot_chain': '2',
    #     'fasta_fp': '../data/input/',
    #     'dhc_fp': '../data/',
    #     'dhc_suffix': '.s1',
    #     'sv_fp_feature': '../data/',
    #     'sv_suffix_feature': '.fs2i1',
    #
    #     'model_frozen_fpn': '../data/model/frozen_graph/s2i1.pb',
    #     'batch_size': 100,
    #     'seq_sep_inferior': 5,
    #     'sv_fp_pred': '../data/',
    #     'sv_suffix_pred': '.s2i1',
    # }
    # rrcontact_stage2(
    #     prot_name=params_s2i1['prot_name'],
    #     prot_chain=params_s2i1['prot_chain'],
    #     fasta_fp=params_s2i1['fasta_fp'],
    #     dhc_fp=params_s2i1['dhc_fp'],
    #     dhc_suffix=params_s2i1['dhc_suffix'],
    #     sv_fp_feature=params_s2i1['sv_fp_feature'],
    #     sv_suffix_feature=params_s2i1['sv_suffix_feature'],
    #     model_frozen_fpn=params_s2i1['model_frozen_fpn'],
    #     sv_fp_pred=params_s2i1['sv_fp_pred'],
    #     sv_suffix_pred=params_s2i1['sv_suffix_pred'],
    #     seq_sep_inferior=params_s2i1['seq_sep_inferior'],
    #     batch_size=100,
    #     verbose=True,
    # )

    # params_s2i2 = {
    #     'prot_name': '2wsc',
    #     'prot_chain': '2',
    #     'fasta_fp': '../data/input/',
    #     'dhc_fp': '../data/',
    #     'dhc_suffix': '.s2i1',
    #     'sv_fp_feature': '../data/',
    #     'sv_suffix_feature': '.fs2i2',
    #
    #     'model_frozen_fpn': '../data/model/frozen_graph/s2i2.pb',
    #     'batch_size': 100,
    #     'seq_sep_inferior': 5,
    #     'sv_fp_pred': '../data/',
    #     'sv_suffix_pred': '.s2i2',
    # }
    # rrcontact_stage2(params=params_s2i2)

    # params_s2i3 = {
    #     'prot_name': '2wsc',
    #     'prot_chain': '2',
    #     'fasta_fp': '../data/input/',
    #     'dhc_fp': '../data/',
    #     'dhc_suffix': '.s2i2',
    #     'sv_fp_feature': '../data/',
    #     'sv_suffix_feature': '.fs2i3',
    #
    #     'model_frozen_fpn': '../data/model/frozen_graph/s2i3.pb',
    #     'batch_size': 100,
    #     'seq_sep_inferior': 5,
    #     'sv_fp_pred': '../data/',
    #     'sv_suffix_pred': '.s2i3',
    # }
    # rrcontact_stage2(params=params_s2i3)

    # params_s2i4 = {
    #     'prot_name': '2wsc',
    #     'prot_chain': '2',
    #     'fasta_fp': '../data/input/',
    #     'dhc_fp': '../data/',
    #     'dhc_suffix': '.s2i3',
    #     'sv_fp_feature': '../data/',
    #     'sv_suffix_feature': '.fs2i4',
    #
    #     'model_frozen_fpn': '../data/model/frozen_graph/s2i4.pb',
    #     'batch_size': 100,
    #     'seq_sep_inferior': 5,
    #     'sv_fp_pred': '../data/',
    #     'sv_suffix_pred': '.s2i4',
    # }
    # rrcontact_stage2(params=params_s2i4)

    params_reformat = {
        'prot_name': '2wsc',
        'prot_chain': '2',
        'file_paths': {
            1: '../data/',
            2: '../data/',
        },
            'file_suffixs': {
            1: '.s2i3',
            2: '.s2i4',
        },
        'tmhmm_fp': '../data/input/',
        'sv_fp': '../data/',
        'format_rr': 'CASP14',
    }
    reformat(
        prot_name=params_reformat['prot_name'],
        prot_chain=params_reformat['prot_chain'],
        file_paths=params_reformat['file_paths'],
        file_suffixs=params_reformat['file_suffixs'],
        tmhmm_fp=params_reformat['tmhmm_fp'],
        sv_fp=params_reformat['sv_fp'],
        format_rr=params_reformat['format_rr'],
    )


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

from itertools import chain
from deephelicon.util.EVCouplingsModel import CouplingsModel

class Plmc:

    def Jij(self, list_2d, position, prot_name, file_chain, param_path):
        pos_ = position
        ec = CouplingsModel(filename=param_path + prot_name + file_chain + '.params')
        abuytts = len(ec.alphabet)
        l0odh = list_2d
        num_pairs = len(pos_)
        for i in range(num_pairs):
            iiuyyy = list(chain.from_iterable(ec.Jij(pos_[i][0], pos_[i][3])))
            for j in range(abuytts * abuytts):
                l0odh[i].append(iiuyyy[j])
        return l0odh


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

from deephelicon.util.Position import Position


class Separation(Position):

    def __init__(self, df, first=None, second=None, is_sort=False, target=None, seq_sep_inferior=None, seq_sep_superior=None):
        super(Separation, self).__init__(seq_sep_inferior, seq_sep_superior)
        self.df = df
        self.seq_sep_inferior = seq_sep_inferior
        self.seq_sep_superior = seq_sep_superior
        self.first = first
        self.second = second
        self.is_sort = is_sort
        self.target = target

    def extract(self):
        df_ = self.df
        if self.seq_sep_inferior is not None and self.seq_sep_superior is None:
            query = (df_[self.second] - df_[self.first] > self.seq_sep_inferior)
        elif self.seq_sep_inferior is None and self.seq_sep_superior is not None:
            query = (df_[self.second] - df_[self.first] < self.seq_sep_superior)
        elif self.seq_sep_inferior is not None and self.seq_sep_superior is not None:
            ss_inf = df_[self.second] - df_[self.first] > self.seq_sep_inferior
            ss_sup = df_[self.second] - df_[self.first] < self.seq_sep_superior
            query = (ss_inf & ss_sup)
        else:
            query = (0 < df_[self.second] - df_[self.first])
        df_ = df_.loc[query].sort_values(
            by=[self.first, self.second],
            ascending=True
        )
        if self.is_sort:
            df_ = df_.loc[query].sort_values([self.target], ascending=False)
        else:
            df_ = df_.loc[query]
        df_ = df_.reset_index(inplace=False, drop=True)
        return df_



__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"


class PFasta:

    def __init__(self, sequence):
        self.sequence = sequence
        self.len_seq = len(self.sequence)

    def pair(self, pos_list):
        seq_dict = self.todict(self.sequence)
        len_pairs = len(pos_list)
        dist_matrix = []
        for i in range(len_pairs):
            fas_id1 = pos_list[i][0]
            fas_id2 = pos_list[i][1]
            dist_matrix.append([
                fas_id1,
                seq_dict[fas_id1],
                fas_id1,
                fas_id2,
                seq_dict[fas_id2],
                fas_id2,
                0
            ])
        return dist_matrix

    def todict(self, seq):
        seq_dict = {}
        len_seq = len(seq)
        for i in range(len_seq):
            seq_dict[i + 1] = seq[i]
        return seq_dict


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import re
import linecache


class TMHMM:

    def read(self, file_path):
        arr = self.format(file_path)
        lpsnp = len(arr)
        tl, tu = [], []
        cl, cu = [], []
        el, eu = [], []
        rl, ru = [], []
        for i in range(lpsnp):
            if arr[i][0] == 'M':
                tl.append(arr[i][1])
                tu.append(arr[i][2])
            elif arr[i][0] == 'i':
                cl.append(arr[i][1])
                cu.append(arr[i][2])
            elif arr[i][0] == 'o':
                el.append(arr[i][1])
                eu.append(arr[i][2])
            else:
                rl.append(arr[i][1])
                ru.append(arr[i][2])
        tmhmm_dict = {
            'cl': cl,
            'cu': cu,
            'tl': tl,
            'tu': tu,
            'el': el,
            'eu': eu,
            'rl': rl,
            'ru': ru,
        }
        return tmhmm_dict

    @classmethod
    def format(cls, file_path):
        line = linecache.getline(file_path + '.tmhmm', 1)
        block1 = re.split(r': ', line)
        block2 = block1[1].split(', ')
        lpsnp = len(block2)
        arr = []
        for i in range(lpsnp):
            tag = block2[i].split(' ')[0]
            beg = block2[i].split(' ')[1]
            end = block2[i].split(' ')[2]
            arr.append([tag, int(beg), int(end)])
        return arr


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import pandas as pd
from deephelicon.util.BPair import BPair
from deephelicon.util.Reader import Reader as pfreader
from deephelicon.util.Separation import Separation as ppssep


class Irreflexive(BPair):

    def __init__(self, sequence=None, window_size=None, window_aa_ids=None, kind='memconp', patch_size=None):
        super(Irreflexive, self).__init__(sequence, window_size, window_aa_ids)
        if kind == 'memconp':
            self.to_dos = [
                [4, -4], [4, 4], [3, -4], [-4, 3], [3, 4],
                [4, 3], [0, -4], [0, 4], [0, -3], [0, 3],
                [-1, 0], [1, 0], [0, 0], [0, -1], [0, 1],
                [3, 0], [-3, 0], [4, 0], [-4, 0], [-3, -4],
                [-4, -3], [-3, 4], [4, -3], [-4, -4], [-4, 4]
            ]
        if kind == 'patch':
            self.to_dos = self.patch(length=patch_size)
        self.num_to_dos = len(self.to_dos)
        self.pfreader = pfreader()

    def pairIds(self):
        n = len(self.sequence)
        num_pending = len(self.to_dos)
        num_aa_in_window = len(self.window_aa_ids[0][0])
        global_pair_ids = [[] for _ in range(self.num_pairs)]
        for i in range(self.num_pairs):
            for j in range(num_aa_in_window):
                if self.window_aa_ids[i][0][j] is None or self.window_aa_ids[i][1][j] is None:
                    for k in range(num_pending):
                        global_pair_ids[i].append([None, None])
                else:
                    for k in range(num_pending):
                        left = self.window_aa_ids[i][0][j] - self.to_dos[k][0]
                        right = self.window_aa_ids[i][1][j] - self.to_dos[k][1]
                        left_inf = left < 1
                        left_sup = left > n
                        right_inf = right < 1
                        right_sup = right > n
                        reflexive = left == right
                        if left_inf or left_sup or right_inf or right_sup or reflexive:
                            global_pair_ids[i].append([None, None])
                        else:
                            if left < right:
                                global_pair_ids[i].append([left, right])
                            else:
                                global_pair_ids[i].append([right, left])
        return global_pair_ids

    def assign(self, list_2d, prot_name, file_chain, dhc_suffix, fc_path=None, cp_path=None, plmc_path=None, gdca_path=None, mi_path=None, dhc_path=None, pair_list_uniform=None):
        list_2d_ = list_2d
        global_pair_ids = self.pairIds()
        pairs_left = []
        pairs_right = []
        for i in range(self.num_pairs):
            for j in range(self.num_to_dos * (2 * self.window_size + 1)):
                pairs_left.append(global_pair_ids[i][j][0])
                pairs_right.append(global_pair_ids[i][j][1])
        mark = len(pairs_left)
        if fc_path is not None:
            evfold_dict = self.freecontact(
                fc_path=fc_path,
                file_name=prot_name,
                file_chain=file_chain,
                sort_=5
            )
            FCj = 0
            for i in range(mark):
                if i % (self.num_to_dos * (2 * self.window_size + 1)) == 0:
                    FCj += 1
                if pairs_left[i] is None or pairs_right[i] is None:
                    list_2d_[FCj - 1].append(0)
                else:
                    inf = pairs_left[i]
                    sup = pairs_right[i]
                    list_2d_[FCj - 1].append(evfold_dict[inf][sup])
        if cp_path is not None:
            ccmpred_dict = self.ccmpred(
                cp_path=cp_path,
                file_name=prot_name,
                file_chain=file_chain,
                sort_=5
            )
            CPj = 0
            for i in range(mark):
                if i % (self.num_to_dos * (2 * self.window_size + 1)) == 0:
                    CPj += 1
                if pairs_left[i] is None or pairs_right[i] is None:
                    list_2d_[CPj - 1].append(0)
                else:
                    inf = pairs_left[i]
                    sup = pairs_right[i]
                    list_2d_[CPj - 1].append(ccmpred_dict[inf][sup])
        if plmc_path is not None:
            plmc_dict = self.plmc(
                plmc_path=plmc_path,
                file_name=prot_name,
                file_chain=file_chain,
                sort_=5
            )
            PCj = 0
            for i in range(mark):
                if i % (self.num_to_dos * (2 * self.window_size + 1)) == 0:
                    PCj += 1
                if pairs_left[i] is None or pairs_right[i] is None:
                    list_2d_[PCj - 1].append(0)
                else:
                    inf = pairs_left[i]
                    sup = pairs_right[i]
                    list_2d_[PCj - 1].append(plmc_dict[inf][sup])
        if gdca_path is not None:
            gdca_dict = self.gdca(
                gdca_path=gdca_path,
                file_name=prot_name,
                file_chain=file_chain,
                sort_=5
            )
            GDCAj = 0
            for i in range(mark):
                if i % (self.num_to_dos * (2 * self.window_size + 1)) == 0:
                    GDCAj += 1
                if pairs_left[i] is None or pairs_right[i] is None:
                    list_2d_[GDCAj - 1].append(0)
                else:
                    inf = pairs_left[i]
                    sup = pairs_right[i]
                    list_2d_[GDCAj - 1].append(gdca_dict[inf][sup])
        if mi_path is not None:
            mi_dict = self.mi(
                mi_path=mi_path,
                file_name=prot_name,
                file_chain=file_chain,
                sort_=5
            )
            MIj = 0
            for i in range(mark):
                if i % (self.num_to_dos * (2 * self.window_size + 1)) == 0:
                    MIj += 1
                if pairs_left[i] is None or pairs_right[i] is None:
                    list_2d_[MIj - 1].append(0)
                else:
                    inf = pairs_left[i]
                    sup = pairs_right[i]
                    list_2d_[MIj - 1].append(mi_dict[inf][sup])
        if dhc_path is not None:
            dhc_dict = self.dhc(
                dhc_path=dhc_path,
                file_name=prot_name,
                file_chain=file_chain,
                file_suffix=dhc_suffix,
                pair_list=pair_list_uniform,
                sort_=5
            )
            TMA165j = 0
            for i in range(mark):
                if i % (self.num_to_dos * (2 * self.window_size + 1)) == 0:
                    TMA165j += 1
                if pairs_left[i] is None or pairs_right[i] is None:
                    list_2d_[TMA165j - 1].append(0)
                else:
                    inf = pairs_left[i]
                    sup = pairs_right[i]
                    list_2d_[TMA165j - 1].append(dhc_dict[inf][sup])
        return list_2d_

    def ttttt(self, arr_2d):
        result = {}
        len_arr = len(arr_2d[0])
        if len_arr == 3:
            for item in arr_2d:
                result.setdefault(item[0], {}).update({item[1]: item[2]})
        else:
            for item in arr_2d:
                result.setdefault(item[0], {}).update({item[1]: item[2:]})
        return result

    def weqd(self, recombine):
        arr_2d = recombine.values.tolist()
        dicts = self.ttttt(arr_2d)
        return dicts

    def mi(self, mi_path, file_name, file_chain, sort_=0):
        self.__sort_ = sort_
        results = self.pfreader.generic(
            mi_path + file_name + file_chain + '.evfold',
            df_sep='\s+',
            is_utf8=True
        )
        results.columns = [
            'contact_id_1',
            'aa_1',
            'contact_id_2',
            'aa_2',
            'score',
            'FC_score'
        ]
        recombine = results[[
            'contact_id_1',
            'contact_id_2',
            'score'
        ]]
        if self.__sort_ == 5:
            recombine_dict = self.weqd(recombine)
            return recombine_dict

    def freecontact(self, fc_path, file_name, file_chain, sort_=0):
        self.__sort_ = sort_
        results = self.pfreader.generic(
            fc_path + file_name + file_chain + '.evfold',
            df_sep='\s+',
            is_utf8=True
        )
        results.columns = [
            'contact_id_1',
            'aa_1',
            'contact_id_2',
            'aa_2',
            'MI_score',
            'score'
        ]
        recombine = results[[
            'contact_id_1',
            'contact_id_2',
            'score'
        ]]
        if self.__sort_ == 5:
            recombine_dict = self.weqd(recombine)
            return recombine_dict

    def ccmpred(self, cp_path, file_name, file_chain, sort_=0):
        self.__sort_ = sort_
        file_results = self.pfreader.generic(
            cp_path + file_name + file_chain + '.ccmpred',
            df_sep='\s+',
            is_utf8=True
        )
        results = []
        for i, row in file_results.iterrows():
            for j in range(file_results.shape[1]):
                if i < j:
                    results.append([i + 1, j + 1, row[j]])
        results = pd.DataFrame(results)
        results.columns = [
            'contact_id_1',
            'contact_id_2',
            'score'
        ]
        recombine = results[[
            'contact_id_1',
            'contact_id_2',
            'score'
        ]]
        if self.__sort_ == 5:
            recombine_dict = self.weqd(recombine)
            return recombine_dict

    def gdca(self, gdca_path, file_name, file_chain, sort_=0):
        self.__sort_ = sort_
        results = self.pfreader.generic(
            gdca_path + file_name + file_chain + '.gdca',
            df_sep='\s+',
            is_utf8=True
        )
        results.columns = [
            'contact_id_1',
            'contact_id_2',
            'score'
        ]
        recombine = results[[
            'contact_id_1',
            'contact_id_2',
            'score'
        ]]
        if self.__sort_ == 5:
            recombine_dict = self.weqd(recombine)
            return recombine_dict

    def plmc(self, plmc_path, file_name, file_chain, sort_=0):
        self.__sort_ = sort_
        results = self.pfreader.generic(
            plmc_path + file_name + file_chain + '.plmc',
            df_sep='\s+',
            is_utf8=True
        )
        results.columns = [
            'contact_id_1',
            'aa_1',
            'contact_id_2',
            'aa_2',
            'placeholder',
            'score'
        ]
        recombine = results[[
            'contact_id_1',
            'contact_id_2',
            'score'
        ]]
        if self.__sort_ == 5:
            recombine_dict = self.weqd(recombine)
            return recombine_dict

    def dhc(self, dhc_path, file_name, file_chain, file_suffix, pair_list=None, sort_=0):
        self.__sort_ = sort_
        results = self.pfreader.generic(
            dhc_path + file_name + file_chain + file_suffix,
            df_sep='\t',
            is_utf8=True
        )
        results.columns = [
            'contact_id_1',
            'aa_1',
            'contact_id_2',
            'aa_2',
            'score'
        ]
        recombine = results[[
            'contact_id_1',
            'contact_id_2',
            'score'
        ]]
        if self.__sort_ == 5:
            recombine = self.sort_3(
                recombine,
                is_sort=False,
                is_uniform=True,
                uniform_df=pd.DataFrame(pair_list),
                indicator=1,
            )
            recombine_dict = self.weqd(recombine)
            return recombine_dict
        elif self.__sort_ == 9:
            return results

    def sort_3(self, recombine, is_sort=False, is_uniform=False, uniform_df=None, indicator=0):
        uconsoo = recombine
        if is_uniform:
            predict_dict = self.weqd(uconsoo)
            uniform_df[2] = indicator
            uniform_list = uniform_df.values.tolist()
            uniform_shape = len(uniform_list)
            for i in range(uniform_shape):
                id_1 = uniform_list[i][0]
                id_2 = uniform_list[i][1]
                try:
                    uniform_list[i][2] = predict_dict[id_1][id_2]
                except KeyError:
                    continue
            uconsoo = pd.DataFrame(uniform_list)
            uconsoo.columns = [
                'contact_id_1',
                'contact_id_2',
                'score'
            ]
        uconsoo = ppssep(
            df=uconsoo,
            first='contact_id_1',
            second='contact_id_2',
            target='score',
            is_sort=is_sort
        ).extract()
        return uconsoo



__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import numpy as np
import pandas as pd


class Segment:

    def __init__(self, seq_sep_inferior=None, seq_sep_superior=None):
        self.seq_sep_inferior = seq_sep_inferior
        self.seq_sep_superior = seq_sep_superior

    def toPair(self, fas_lower, fas_upper):
        if fas_lower == [] and fas_upper == []:
            return []
        else:
            df_ = pd.DataFrame(self.iudsna(fas_lower, fas_upper))
            pairs = self.extract(
                df=df_,
                first=0,
                second=1,
            ).values.tolist()
            return pairs

    def extract(self, df, first, second):
        df_ = df
        if self.seq_sep_inferior is not None and self.seq_sep_superior is None:
            query = (df_[second] - df_[first] > self.seq_sep_inferior)
        elif self.seq_sep_inferior is None and self.seq_sep_superior is not None:
            query = (df_[second] - df_[first] < self.seq_sep_superior)
        elif self.seq_sep_inferior is not None and self.seq_sep_superior is not None:
            ss_inf = df_[second] - df_[first] > self.seq_sep_inferior
            ss_sup = df_[second] - df_[first] < self.seq_sep_superior
            query = (ss_inf & ss_sup)
        else:
            query = (0 < df_[second] - df_[first])
        df_ = df_.loc[query].sort_values(
            by=[first, second],
            ascending=True
        )
        df_ = df_.loc[query]
        df_ = df_.reset_index(inplace=False, drop=True)
        return df_

    def iudsna(self, inf_arr, sup_arr):
        conerants2 = []
        p8 = len(inf_arr)
        for i in range(p8):
            conerants2.append(list(np.arange(inf_arr[i], sup_arr[i] + 1)))
        iutucy = []
        for i in range(p8):
            for j in range(p8):
                if i < j:
                    for p in range(len(conerants2[i])):
                        for q in range(len(conerants2[j])):
                            iutucy.append([conerants2[i][p], conerants2[j][q]])
        return iutucy


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import pandas as pd


class Reader:

    def generic(self, df_fpn, df_sep='\t', header=None, is_utf8=False):
        if is_utf8:
            return pd.read_csv(df_fpn, sep=df_sep, header=header, encoding='utf-8')
        else:
            return pd.read_csv(df_fpn, sep=df_sep, header=header)


__author__ = "Jianfeng Sun"
__version__ = "0.0.1"
__copyright__ = "Copyright 2025"
__license__ = "GPL-3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import pandas as pd
from deephelicon.util.Irreflexive import Irreflexive
from deephelicon.util.TMHMM import TMHMM
from deephelicon.util.Segment import Segment
from deephelicon.util.Console import Console


class Reformat:

    def __init__(
            self,
            verbose=True,
    ):
        self.idr = Irreflexive(None, 2, [-1, -1])

        self.console = Console()
        self.console.verbose = verbose

    def run(
            self,
            prot_name,
            file_chain,
            tmhmm_fp,
            file_paths,
            file_suffixs,
            file_ids,
            sv_fp,
            format_rr='CASP14',
    ):
        self.console.print("===>Protein: {} chain: {}".format(prot_name, file_chain))
        self.console.print("======>Reformatting...")
        files = {}
        fps_l = file_paths.split(";")
        fss_l = file_suffixs.split(";")
        fids_l = file_ids.split(";")
        # for k, v in file_paths.items():
        for k, v in enumerate(fps_l):
            files[int(fids_l[k])] = self.idr.dhc(
                dhc_path=v,
                file_name=prot_name,
                file_chain=file_chain,
                file_suffix=fss_l[k],
                sort_=9
            )
        if format_rr == 'CASP14':
            annotation = files[1][[
                'contact_id_1',
                'contact_id_2',
            ]]
        else:
            annotation = files[1][[
                'contact_id_1',
                'aa_1',
                'contact_id_2',
                'aa_2',
            ]]
        for k, each in files.items():
            files[k] = each.rename(columns={'score': 'score' + str(k)})
        each_combine = pd.concat(
            [each['score' + str(u)] for u, each in files.items()],
            axis=1
        )
        mean_each = 0.6 * each_combine['score1'] + 0.4 * each_combine['score2']
        # mean_each = each_combine.mean(axis=1)
        ensemble_ = pd.concat([annotation, mean_each], axis=1)
        ensemble_[0] = ensemble_[0].astype(float)
        # print(ensemble_)
        l = TMHMM().read(file_path=tmhmm_fp + prot_name + file_chain)['tl']
        u = TMHMM().read(file_path=tmhmm_fp + prot_name + file_chain)['tu']
        if len(l) >= 2:
            pos_list = Segment().toPair(l, u)
            ts = pd.DataFrame()
            asdasd = ensemble_.values.tolist()
            for tt, i in enumerate(pos_list):
                for rr, j in enumerate(asdasd):
                    if format_rr == 'CASP14':
                        if j[0] == i[0] and j[1] == i[1]:
                            ts[tt] = ensemble_.iloc[rr]
                    else:
                        if j[0] == i[0] and j[2] == i[1]:
                            ts[tt] = ensemble_.iloc[rr]
            final_ts = ts.T
            # print(final_ts)
            if format_rr == 'CASP14':
                final_ts = final_ts.sort_values(by=[0], ascending=False).reset_index(drop=True)
                final_ts[0] = final_ts[0].apply(lambda x: '{:.3f}'.format(x))
                final_ts[0] = final_ts[0].astype(float)
                final_ts[0] = final_ts[0].apply(lambda x: '{:g}'.format(x))
                final_ts[0] = final_ts[0].apply(lambda x: x.replace('0.', '.'))
                final_ts['contact_id_1'] = final_ts['contact_id_1'].astype(int)
                final_ts['contact_id_2'] = final_ts['contact_id_2'].astype(int)
                final_ts.to_csv(
                    sv_fp + prot_name + file_chain + '.deephelicon',
                    sep=' ',
                    header=False,
                    index=False,
                )
            else:
                final_ts.to_csv(
                    sv_fp + prot_name + file_chain + '.deephelicon',
                    sep='\t',
                    header=False,
                    index=False,
                )
            return final_ts
        else:
            ensemble_.to_csv(
                sv_fp + prot_name + file_chain + '.deephelicon',
                sep='\t',
                header=False,
                index=False,
            )
            return ensemble_